/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

Contrôle de SP0256-AL2
Conçu pour rouler sur un Arduino-Uno
Électro-Bidouilleur - Mars 2020
*/

// Définition des broches vers le SP0256-AL2
#define A1 2
#define A2 3
#define A3 4
#define A4 5
#define A5 6
#define A6 7
#define ALDN 8 
#define SBY 9

/* Exemples de phrases
 EY,LL,EH,KK1,TT1,RR2,OW,BB2,IY,DD1,UW2,YY2,AX,AO,AR,PA5,                    Électro-Bidouilleur
 BB2,IY,DD1,UW2,YY2,AX,AO,AR,PA4,SS,SS,AE,AE,LL,UW2,PA5,                     Bidouilleurs, salut
 WW,AO,NN1,PA5,TT2,UW2,PA5,TH,RR2,IY,PA5,FF,FF,OR,PA5,FF,FF,AY,VV,PA5,       1 2 3 4 5
 HH1,EH,EH,LL,AX,OW,PA4,WW,ER1,ER1,LL,PA2,DD1,PA5,                           Hello World
 BB2,OW,NN1,ZH,UW2,RR2,PA4,LL,AX,PA4,MM,OW,NN1,DD2,PA5,                      Bonjour le monde
 AE,AE,LL,AE,PP,RR2,AO,SH,EH,EH,NN1,PA5,                                     À la prochaine
*/
// La liste des allophones. Leur index dans la liste (array) constitue le code à envoyer au SP0256-AL2
String alloph_list[65] = {"PA1","PA2","PA3","PA4","PA5","OY","AY","EH",
                          "KK3","PP","JH","NN1","IH","TT2","RR1","AX","MM",
                          "TT1","DH1","IY","EY","DD1","UW1","AO","AA","YY2",
                          "AE","HH1","BB1","TH","UH","UW2","AW","DD2","GG3",
                          "VV","GG1","SH","ZH","RR2","FF","KK2","KK1","ZZ",
                          "NG","LL","WW","XR","WH","YY1","CH","ER1","ER2",
                          "OW","DH2","SS","NN2","HH2","OR","AR","YR","GG2",
                          "EL","BB2",""}; 

// La fonction d'initialisation de l'Arduino. Exécutée uen seule fois.
void setup() 
{
  Serial.begin(115200);           // Configuration du port série 
  Serial.setTimeout(10000);

  digitalWrite(ALDN,HIGH);        // Configuration des broches d'entrée/sortie
  pinMode(A1, OUTPUT);    
  pinMode(A2, OUTPUT);   
  pinMode(A3, OUTPUT);    
  pinMode(A4, OUTPUT);    
  pinMode(A5, OUTPUT);   
  pinMode(A6, OUTPUT);    
  pinMode(ALDN, OUTPUT);    
  pinMode(SBY, INPUT);    
}

// La fonction bouclante de l'Arduino. Exécutée à répétition.
void loop() 
{
// Les variable de la fonction  
int i;
char RxChar;
String RxStr;
int ctr;
int num_phonemes;
int alloph[100] = {};

  RxStr = "";
  ctr= 0;
  num_phonemes = 0;
  
  while (true)      
  {   
    if (Serial.available()>0)   // Caractères disponibles dans la mémoire-tampon du port série?
    {     // Oui
      RxChar = Serial.read();                     // Lire le caractère
      if (RxChar == '\n') break;                  // Fin de la chaîne de caractères reçus
      if (RxChar != ',') RxStr += RxChar;         // Le transférer dans la chaîne de caractère en RAM
      else
      {
        i = 0;
        while ((alloph_list[i] != RxStr) && (i < 64)) i++; // Recherche du Phonème dans la liste
        if (i == 64)          // Phonème pas trouvé dans la liste
        { 
          Serial.print("Erreur avec '");          // Afficher un message d'erreur
          Serial.print(RxStr);
          Serial.println("'");        
        }
        else      // Phonème trouvé dans la liste
        {
          alloph[num_phonemes] = i;   // Transférer la position du phonème de la liste comme code de phonème
          num_phonemes++;             // Incréementer la position du phonème dans la phrase à envoyer 
        }
        RxStr = "";
      }
    }
  }
  // Tous les phonèmes sont reçus. On peut produire la phrase
  for (i=0;i<num_phonemes;i++)  // Boucle d'incrémentation des phonèmes à prononcer
  {
    PORTD = (PORTD & B00000011) | (alloph[i]<<2);   // Transférer le phonème sur le port vers le SP0256
    digitalWrite(ALDN,LOW);         // donner une impulsion basse sur la ligne Address Load
    delayMicroseconds(1);
    digitalWrite(ALDN,HIGH);    
    Serial.println(alloph[i]);      // Afficher le code du phonème
    while (!digitalRead(SBY));      // Attendre que la ligne Standby remonte avant d'envoyer le phonème suivant
  }
}
